@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 2.4.0+   2011/09/29
+ * Version: 2.5.0   2011/09/29
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License v2 as published by the
@@ -99,6 +99,9 @@ static void ccs_check_condition(char *condition)
 		CCS_MODE_OTHERS_READ,     /* S_IROTH */
 		CCS_MODE_OTHERS_WRITE,    /* S_IWOTH */
 		CCS_MODE_OTHERS_EXECUTE,  /* S_IXOTH */
+		CCS_TASK_TYPE,            /* ((u8) task->ccs_flags) &
+					     CCS_TASK_IS_EXECUTE_HANDLER */
+		CCS_TASK_EXECUTE_HANDLER, /* CCS_TASK_IS_EXECUTE_HANDLER */
 		CCS_EXEC_REALPATH,
 		CCS_SYMLINK_TARGET,
 		CCS_PATH1_UID,
@@ -165,6 +168,8 @@ static void ccs_check_condition(char *condition)
 		[CCS_MODE_OTHERS_READ]     = "others_read",
 		[CCS_MODE_OTHERS_WRITE]    = "others_write",
 		[CCS_MODE_OTHERS_EXECUTE]  = "others_execute",
+		[CCS_TASK_TYPE]            = "task.type",
+		[CCS_TASK_EXECUTE_HANDLER] = "execute_handler",
 		[CCS_EXEC_REALPATH]        = "exec.realpath",
 		[CCS_SYMLINK_TARGET]       = "symlink.target",
 		[CCS_PATH1_UID]            = "path1.uid",
@@ -346,6 +351,48 @@ static _Bool ccs_check_domain(char *arg)
 	return ccs_prune_word(arg, cp);
 }
 
+static _Bool ccs_check_transition(char *arg)
+{
+	char *cp;
+	_Bool conflict = strstr(arg, " auto_domain_transition=") != NULL;
+	if (*arg == '<')
+		return !conflict && ccs_check_domain(arg);
+	cp = strchr(arg, ' ');
+	if (cp)
+		*cp++ = '\0';
+	if (ccs_correct_path(arg) || !strcmp(arg, "keep") ||
+	    !strcmp(arg, "initialize") || !strcmp(arg, "reset") ||
+	    !strcmp(arg, "child") || !strcmp(arg, "parent"))
+		return !conflict && ccs_prune_word(arg, cp);
+	if (cp)
+		*--cp = ' ';
+	return true;
+}
+
+static _Bool ccs_check_path_transition(char *arg)
+{
+	if (!ccs_check_path(arg))
+		return false;
+	return ccs_check_transition(arg);
+}
+
+static _Bool ccs_check_capability(char *arg)
+{
+	static const char * const list[] = {
+		"use_route", "use_packet", "SYS_REBOOT", "SYS_VHANGUP",
+		"SYS_TIME", "SYS_NICE", "SYS_SETHOSTNAME", "use_kernel_module",
+		"SYS_KEXEC_LOAD", "SYS_PTRACE", NULL
+	};
+	int i;
+	char *cp = strchr(arg, ' ');
+	if (cp)
+		*cp++ = '\0';
+	for (i = 0; list[i]; i++)
+		if (!strcmp(arg, list[i]))
+			return ccs_prune_word(arg, cp);
+	return false;
+}
+
 static _Bool ccs_check_u8(char *arg)
 {
 	char *cp = strchr(arg, ' ');
@@ -357,6 +404,113 @@ static _Bool ccs_check_u8(char *arg)
 	return ccs_prune_word(arg, cp);
 }
 
+static _Bool ccs_check_u16(char *arg)
+{
+	char *cp = strchr(arg, ' ');
+	unsigned int value;
+	if (cp)
+		*cp++ = '\0';
+	if (sscanf(arg, "%u", &value) != 1 || value >= 65536)
+		return false;
+	return ccs_prune_word(arg, cp);
+}
+
+static _Bool ccs_check_ip_address(char *arg)
+{
+	char *cp = strchr(arg, ' ');
+	struct ccs_ip_address_entry entry = { };
+	if (cp)
+		*cp++ = '\0';
+	if (*arg == '@') /* Don't reject address_group. */
+		goto found;
+	if (ccs_parse_ip(arg, &entry) ||
+	    memcmp(entry.min, entry.max, 16) > 0)
+		return false;
+found:
+	return ccs_prune_word(arg, cp);
+}
+
+static _Bool ccs_check_port(char *arg)
+{
+	char *cp = strchr(arg, ' ');
+	unsigned int min_value;
+	unsigned int max_value;
+	if (cp)
+		*cp++ = '\0';
+	switch (sscanf(arg, "%u-%u", &min_value, &max_value)) {
+	case 2:
+	case 1:
+		break;
+	default:
+		return false;
+	}
+	return ccs_prune_word(arg, cp);
+}
+
+
+static _Bool ccs_check_network(char *arg)
+{
+	static const struct {
+		const char *directive;
+		u8 flags;
+	} list[] = {
+		{ "bind", 1 },
+		{ "listen", 2 },
+		{ "connect", 2 },
+		{ "accept", 2 },
+		{ "send", 4 },
+		{ "recv", 4 },
+	};
+	_Bool inet;
+	u8 mask;
+	u8 flags = 0;
+	char *start = arg;
+	if (ccs_str_starts(arg, "inet "))
+		inet = true;
+	else if (ccs_str_starts(arg, "unix "))
+		inet = false;
+	else
+		return false;
+	if ((inet && ccs_str_starts(arg, "stream ")) ||
+	    (!inet && (ccs_str_starts(arg, "stream ") ||
+		       ccs_str_starts(arg, "seqpacket "))))
+		mask = 2;
+	else if ((inet && (ccs_str_starts(arg, "dgram ") ||
+			   ccs_str_starts(arg, "raw "))) ||
+		 (!inet && ccs_str_starts(arg, "dgram ")))
+		mask = 4;
+	else
+		return false;
+	while (1) {
+		u8 type;
+		while (ccs_str_starts(arg, "/"));
+		if (ccs_str_starts(arg, " "))
+			break;
+		for (type = 0; list[type].directive; type++) {
+			if (((list[type].flags | mask) & 6) == 6)
+				continue;
+			if (!ccs_str_starts(arg, list[type].directive))
+				continue;
+			flags |= list[type].flags;
+			break;
+		}
+		if (!list[type].directive) {
+			while (*arg && *arg != ' ' && *arg != '/')
+				arg++;
+			*arg = '\0';
+			goto out;
+		}
+	}
+	if (!flags)
+		goto out;
+	ccs_prune_word(start, arg);
+	return inet ? ccs_check_ip_address(start) && ccs_check_number(start) :
+		ccs_check_path(start);
+out:
+	return false;
+
+}
+
 static _Bool ccs_check_path_domain(char *arg)
 {
 	if (!strncmp(arg, "any ", 4))
@@ -415,7 +569,7 @@ static _Bool ccs_check_file(char *arg)
 		{ "chown", ccs_check_path_number },
 		{ "chroot", ccs_check_path },
 		{ "create", ccs_check_path_number },
-		{ "execute", ccs_check_path },
+		{ "execute", ccs_check_path_transition },
 		{ "getattr", ccs_check_path },
 		{ "ioctl", ccs_check_path_number },
 		{ "link", ccs_check_path2 },
@@ -473,7 +627,15 @@ static _Bool ccs_check_domain_policy2(char *policy)
 		_Bool (*arg1) (char *arg);
 		_Bool (*arg2) (char *arg);
 	} list[] = {
+		{ "capability ", ccs_check_capability },
 		{ "file ", ccs_check_file },
+		{ "ipc signal ", ccs_check_u16, ccs_check_domain },
+		{ "misc env ", ccs_check_path },
+		{ "network ", ccs_check_network },
+		{ "task auto_domain_transition ", ccs_check_domain },
+		{ "task auto_execute_handler ", ccs_check_path_transition },
+		{ "task denied_execute_handler ", ccs_check_path_transition },
+		{ "task manual_domain_transition ", ccs_check_domain },
 		{ }
 	};
 	for (type = 0; list[type].directive; type++) {
@@ -527,7 +689,9 @@ static void ccs_check_exception_policy(char *policy)
 		_Bool (*arg2) (char *arg);
 	} list[] = {
 		{ "acl_group ", ccs_check_u8, ccs_check_domain_policy2 },
+		{ "address_group ", ccs_check_path, ccs_check_ip_address },
 		{ "aggregator ", ccs_check_path, ccs_check_path },
+		{ "deny_autobind ", ccs_check_port },
 		{ "initialize_domain ", ccs_check_path_domain },
 		{ "keep_domain ", ccs_check_path_domain },
 		{ "no_initialize_domain ", ccs_check_path_domain },
