@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 2.4.0+   2014/06/01
+ * Version: 2.5.0   2011/09/29
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License v2 as published by the
@@ -43,7 +43,6 @@
 #include <stdarg.h>
 #include <sched.h>
 #include <sys/mount.h>
-#include <poll.h>
 
 #define s8 __s8
 #define u8 __u8
@@ -90,17 +89,9 @@ struct ccs_number_entry {
 
 struct ccs_domain_info {
 	const struct ccs_path_info *domainname;
-	const char *target_domainname; /* This may be NULL */
-	const struct ccs_transition_control_entry *d_t; /* This may be NULL */
 	const struct ccs_path_info **string_ptr;
 	int string_count;
-	int number;   /* domain number (-1 if is_dis or is_dd) */
 	u8 profile;
-	_Bool is_dis; /* domain initializer source */
-	_Bool is_dit; /* domain initializer target */
-	_Bool is_dk;  /* domain keeper */
-	_Bool is_du;  /* unreachable domain */
-	_Bool is_dd;  /* deleted domain */
 	_Bool profile_assigned;
 	u8 group;
 };
@@ -151,16 +142,15 @@ const char *ccs_domain_name(const struct ccs_domain_policy *dp,
 const struct ccs_path_info *ccs_savename(const char *name);
 int ccs_add_string_entry(struct ccs_domain_policy *dp, const char *entry,
 			 const int index);
-int ccs_assign_domain(struct ccs_domain_policy *dp, const char *domainname,
-		      const _Bool is_dis, const _Bool is_dd);
+int ccs_assign_domain(struct ccs_domain_policy *dp, const char *domainname);
 int ccs_del_string_entry(struct ccs_domain_policy *dp, const char *entry,
 			 const int index);
 int ccs_find_domain(const struct ccs_domain_policy *dp,
-		    const char *domainname0, const _Bool is_dis,
-		    const _Bool is_dd);
+		    const char *domainname0);
 int ccs_find_domain_by_ptr(struct ccs_domain_policy *dp,
 			   const struct ccs_path_info *domainname);
 int ccs_open_stream(const char *filename);
+int ccs_parse_ip(const char *address, struct ccs_ip_address_entry *entry);
 int ccs_parse_number(const char *number, struct ccs_number_entry *entry);
 int ccs_string_compare(const void *a, const void *b);
 int ccs_write_domain_policy(struct ccs_domain_policy *dp, const int fd);
